/*
    qtparted - a frontend to libparted for manipulating disk partitions
    Copyright (C) 2002-2003 Vanni Brutto

    Vanni Brutto <zanac (-at-) libero dot it>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <qapplication.h>
#include <stdlib.h>
#include <sys/param.h>  // MAXPATHLEN
#include <sys/stat.h>   // S_ISLNK
#include <unistd.h>     // readlink
#include <dirent.h>
#include "qp_libparted.h"
#include "qp_fswrap.h"
#include "qp_common.h"

#define MIN_FREESPACE		(1000 * 2)	/* 1000k */
#define PED_ASSERT(cond, action)        while (0) {}

#define SUCCESS tr("Operation successfully completed.")
#define NOTSUCCESS tr("An error has occurs during the operation.")
#define ERROR_PED_DISK_NEW tr("An error has happened during the ped_disk_new call.")
#define ERROR_PED_DISK_GET_PARTITION tr("An error has happened during the ped_disk_get_partition call.")
#define ERROR_PED_PARTITION_NEW tr("An error has happened during ped_partition_new call.")
#define ERROR_PED_DISK_ADD_PARTITION tr("An error has happened during ped_disk_add_partition call.")
#define ERROR_PED_PARTITION_SET_SYSTEM tr("An error has happened during ped_partition_set_system call.")
#define ERROR_PED_PARTITION_SET_FLAG tr("An error has happened during ped_partition_set_flag call.")

QString MB2String(float mbyte) {
    QString label;

    /*---use the size in MByte or in GByte---*/
    if (mbyte > (1024 * 1024))   label.sprintf("%3.2fTB", mbyte / (1024 * 1024));
    else if (mbyte > 1024)       label.sprintf("%3.2fGB", mbyte / 1024);
    else                         label.sprintf("%3.2fMB", mbyte);

    return label;
}

/*-begin of QP_PartInfo--------------------------------------------------------------------------*/
float QP_PartInfo::mb_t_start() {
    return t_start * 1.0 / MEGABYTE_SECTORS;
}

float QP_PartInfo::mb_t_end() {
    return t_end * 1.0 / MEGABYTE_SECTORS;
}

float QP_PartInfo::mb_start() {
    return start * 1.0 / MEGABYTE_SECTORS;
}

float QP_PartInfo::mb_end() {
    return end * 1.0 / MEGABYTE_SECTORS;
}

float QP_PartInfo::mb_min_size() {
    return min_size * 1.0 / MEGABYTE_SECTORS;
}

QString QP_PartInfo::partname() {
    QString x = QString("%1%2") .arg(device) .arg(num);
    return x;
}

bool QP_PartInfo::isFree() {
    return (fsspec == _free);
}

bool QP_PartInfo::isUnknow() {
    return (fsspec == _unknow);
}

bool QP_PartInfo::isActive() {
    return _active;
}

bool QP_PartInfo::canBeActive() {
    return _canBeActive;
}

bool QP_PartInfo::setActive(bool active) {
    return _libparted->partition_set_flag_active(this, active);
}
    
bool QP_PartInfo::resize(PedSector new_start, PedSector new_end) {
    /*---test if the partition is busy (ie mounted)---*/
//    if (partition_is_busy())
//        return false;

    /*---does it exist a wrapper for this filesystem?---*/
    if (fsspec->fswrap()) {
        /*---does it exist a resize wrapper?                ---
         *---please note that this test should be a surplus!---*/
        if (fsspec->fswrap()->wrap_resize) {
            bool rc;
            /*---the user want to shrink or enlarge the partition?---*/

            /*---the user want to shrink!---*/
            if (new_end < end) {
                /*---update the filesystem---*/
                rc = fsspec->fswrap()->resize(partname(), new_end - new_start);
                if (!rc) {
                    _libparted->_message = fsspec->fswrap()->message();
                    _libparted->emitSigTimer(100, _libparted->message(), QString::null);
                    return false;
                }

                /*---and now update geometry of the partition---*/
                rc = _libparted->set_geometry(this, new_start, new_end + 4*MEGABYTE_SECTORS);
                if (!rc) {
                    _libparted->emitSigTimer(100, _libparted->message(), QString::null);
                    return false;
                } else {
                    _libparted->emitSigTimer(100, _libparted->message(), QString::null);
                    return true;
                }
            /*---the user want to enlarge!---*/
            } else {
                /*---first se the geometry of the partition---*/
                if (!_libparted->set_geometry(this, new_start, new_end + 4*MEGABYTE_SECTORS)) {
                    _libparted->emitSigTimer(100, _libparted->message(), QString::null);
                    return false;
                }
                /*---and now update the filesystem!---*/
                if (!fsspec->fswrap()->resize(partname(), new_end - new_start)) {
                    _libparted->_message = fsspec->fswrap()->message();
                    _libparted->emitSigTimer(100, _libparted->message(), QString::null);
                    return false;
                } else {
                    _libparted->emitSigTimer(100, SUCCESS, QString::null);
                    return true;
                }
            }
        }
    } else {
        /*---the filesystem is supported by libparted, :)---*/
        bool rc = _libparted->resize(this, new_start, new_end);
        if (!rc) {
            _libparted->emitSigTimer(100, _libparted->message(), QString::null);
        } else {
            _libparted->emitSigTimer(100, SUCCESS, QString::null);
        }
        return rc;
    }

    _libparted->emitSigTimer(100, NOTSUCCESS, QString::null);
    return false;
}

bool QP_PartInfo::move(PedSector new_start, PedSector new_end) {
    bool rc = _libparted->move(this, new_start, new_end);
    if (!rc) {
        _libparted->emitSigTimer(100, _libparted->message(), QString::null);
    } else {
        _libparted->emitSigTimer(100, SUCCESS, QString::null);
    }
    return rc;
}

bool QP_PartInfo::partition_is_busy() {
    return _libparted->partition_is_busy(this->num);
}

bool QP_PartInfo::set_system(QP_FileSystemSpec *fsspec) {
    return _libparted->set_system(this, fsspec);
}

bool QP_PartInfo::fswrap() {
    return fsspec->fswrap();
}

PedSector QP_PartInfo::wrap_min_size() {
    if (fsspec->fswrap())
        return fsspec->fswrap()->wrap_min_size;
    else
        return min_size;
}

QString QP_PartInfo::message() {
    return fsspec->fswrap()->message();
}
/*-end of QP_PartInfo----------------------------------------------------------------------------*/





/*-begin of QP_LibParted-------------------------------------------------------------------------*/


/*structur used for the update of the "time left" on progress bar*/
typedef struct {
	time_t	last_update;
	time_t	predicted_time_left;
    QP_LibParted *libparted;
} TimerContext;

static PedTimer *timer;
static TimerContext timer_context;

//function called by libparted. 'Cause libparted is write in C you cannot use a method!
void _timer_handler(PedTimer *timer, void *context) {
    TimerContext *tcontext = (TimerContext *)context;
    int draw_this_time;

    if (tcontext->last_update != timer->now && timer->now > timer->start) {
        tcontext->predicted_time_left = timer->predicted_end - timer->now;
        tcontext->last_update = timer->now;
        draw_this_time = 1;
    } else {
        draw_this_time = 0;
    }

    if (draw_this_time) {
        if (timer->state_name) {
            int percent = (int)(100.0 * timer->frac);
            QString state = QString(timer->state_name);
            QString timeleft;
            timeleft.sprintf("%.2d:%.2d",
                             (int)(tcontext->predicted_time_left / 60),
                             (int)(tcontext->predicted_time_left % 60));
            tcontext->libparted->emitSigTimer(percent, state, timeleft);
        }
    }
}


QP_LibParted::QP_LibParted(QString device) {
    /*---prevent from memory leak: when list are cleared destroy partinfo object!---*/
    logilist.setAutoDelete(true);
    partlist.setAutoDelete(true);

    /*---get all filesystem supported by libparted---*/
    filesystem = new QP_FileSystem();
    get_filesystem(filesystem);

    dev = NULL;
    /*---initialize active partition---*/
    _partActive = NULL;

    _device = device;
    _FastScan = true;
}


QP_LibParted::~QP_LibParted() {
    /*TODO FIXME static void
    _done (PedDevice* dev)   in destroy!
    */
	ped_timer_destroy (timer);

    logilist.clear();
    partlist.clear();

    delete filesystem;
    if (dev) ped_device_destroy(dev);
}


void QP_LibParted::scan_partitions() {
    /*---initialize active partition---*/
    _partActive = NULL;

    _message = QString::null;
    if (_device.isNull()) return ;
    const char *szdevice = _device.latin1();

    dev = ped_device_get(szdevice);
    if (dev == NULL) {
        printf("error!\n");
        exit(1);
    }
    _mb_hdsize = (dev->length * dev->sector_size) / MEGABYTE;

    _message = QString(tr("Scanning all disk partitions."));

    /*tacc*/
    timer_context.libparted = this;
	timer = ped_timer_new (_timer_handler, &timer_context);
    if (!timer) printf("no timer!\n");
    timer_context.last_update = 0;
    

    PedDisk *disk = NULL;
    disk = ped_disk_new(dev);
    if (!disk) {
        _message = QString(ERROR_PED_DISK_NEW);
        //there aren't partition!
        printf("errore!\n");
        exit(1);
    }

    has_extended = false;
	//int has_name;
	//int _has_extended = ped_disk_type_check_feature (disk->type,
	//		       		 PED_DISK_TYPE_EXTENDED);
	//has_name = ped_disk_type_check_feature (disk->type,
	//		       		 PED_DISK_TYPE_PARTITION_NAME);

    PedConstraint* constraint = NULL;
	PedPartition*	part = NULL;
	for (part = ped_disk_next_partition (disk, NULL); part;
	     part = ped_disk_next_partition (disk, part)) {

        if (part->type & PED_PARTITION_METADATA) //FIXME what is METADATA?
            continue;

        QP_PartInfo *partinfo = new QP_PartInfo();
        partinfo->start = part->geom.start;
        partinfo->end = part->geom.end;
        partinfo->num = part->num;
        partinfo->device = _device;
        partinfo->min_size = -1;
        partinfo->label = QString::null;
        partinfo->_free = filesystem->free();
        partinfo->_unknow = filesystem->unknow();
        partinfo->_libparted = this;
        partinfo->_active = false;
        partinfo->_canBeActive = false;

        if (part->type & PED_PARTITION_LOGICAL)
            partinfo->type = QTParted::logical;
        else if (part->type & PED_PARTITION_EXTENDED) {
            partinfo->type = QTParted::extended;
            has_extended = true;
        }
        else partinfo->type = QTParted::primary;
                         
        const char *part_filesystem = part->fs_type ? part->fs_type->name : "";

        if (part->type & PED_PARTITION_FREESPACE) {
            partinfo->fsspec = filesystem->free();
            partinfo->t_start = -1;
            partinfo->t_end = -1;
        } else {
            partition_get_flags(partinfo, part);
            partinfo->fsspec = filesystem->nameToFSSpec(part_filesystem); //fat16, fat32 etc etc
            constraint = ped_constraint_any(dev);
            PedGeometry *geometry =
                ped_disk_get_max_partition_geometry(disk, part, constraint);
            if (geometry != NULL) {
                partinfo->t_start = geometry->start;
                partinfo->t_end = geometry->end;
            } else {
                partinfo->t_start = -1;
                partinfo->t_end = -1;
            }
        }

        if (partinfo->type == QTParted::logical)
             logilist.append(partinfo);
        else partlist.append(partinfo);
    }

    /*---count how partition are in the disk---*/
    int totPart = partlist.count() + logilist.count();
    
    /*---loop for all partition of the disk---*/
    QP_PartInfo *p;
    int i = 0;
    for (p = (QP_PartInfo*)partlist.first(); p; p = (QP_PartInfo*)partlist.next()) {
        if (p->type != QTParted::extended) {
            i++;
            if (!p->isFree()) {
                /*---emit a signal for update the progressbar---*/
                _message = QString(tr("Getting info about partition %1."))
                                   .arg(p->partname());
                emitSigTimer(i*100/totPart, message(), QString::null);

                /*---get info about this primary partition---*/
                part = ped_disk_get_partition(disk, p->num);
                if (part) get_partfilesystem_info(part, p);
            }
        } else {
            QP_PartInfo *logi;
            /*---loop for every logical partitions---*/
            for (logi = (QP_PartInfo*)logilist.first(); logi; logi = (QP_PartInfo*)logilist.next()) {
                i++;
                if (!logi->isFree()) {
                    /*---emit a signal for update the progressbar---*/
                    _message = QString(tr("Getting info about partition %1."))
                                       .arg(logi->partname());
                    emitSigTimer(i*100/totPart, message(), QString::null);
                
                    /*---get info about this logical partition---*/
                    part = ped_disk_get_partition(disk, logi->num);
                    if (part) get_partfilesystem_info(part, logi);
                }
            }
        }
    }
    
    emitSigTimer(100, message(), QString::null);
    ped_disk_destroy(disk);
}

QStrList QP_LibParted::device_probe() {
    QStrList tmp;

    PedDevice *dev = NULL;
    ped_device_probe_all();
    while((dev = ped_device_get_next(dev))) {
      if ( dev->type == PED_DEVICE_IDE )
        tmp.append(dev->path);
        //addDevice( dev, ideRoot );
      else if ( dev->type == PED_DEVICE_SCSI )
        tmp.append(dev->path);
        //addDevice( dev, scsiRoot );
      else if ( dev->type == PED_DEVICE_DAC960 )
        tmp.append(dev->path);
        //addDevice( dev, dacRoot );
      else
        //printf("non so\n");
        tmp.append(dev->path);
        //addDevice( dev, unknownRoot );
    };

    return tmp;
}


qtp_DriveInfo QP_LibParted::device_info(QString strdev) {
    qtp_DriveInfo driveinfo;
    
    PedDevice *dev = ped_device_get((const char *)strdev.latin1());

    long double length = ( dev->length * (dev->sector_size / 1024.0) ) /1024.0;
  
    driveinfo.device = strdev;
    driveinfo.model = dev->model;

    char str[30];
    sprintf(str, "%Lg", length);
    driveinfo.mb_capacity = str;

    sprintf(str, "%lld", dev->length);
    driveinfo.length_sectors = str;

    return driveinfo;
}

/*---this function convert a long devfs device name to a good old device name (short)
 *   the code was bring from partimage software made by Franois Dupoux---*/
int QP_LibParted::convertDevfsNameToShortName(const char *szDevfs, char *szShort, int nMaxShort) {
    DIR *fdDir;
    struct dirent *dire;
    struct stat st;
    int nRes;
    char szTemp[MAXPATHLEN];
    char szTemp1[MAXPATHLEN];
    char szTemp2[MAXPATHLEN];
    int i, dwSlash;

    // in case of error, use this file:
    snprintf(szShort, nMaxShort, "%s", szDevfs);

    for (i=0, dwSlash=0; szDevfs[i]; i++)
       if (szDevfs[i] == '/')
           dwSlash++;

    //printf ("Slash(%s)=%d\n", szDevfs, (int)dwSlash);

    // exit if not a devfs name
    if (dwSlash <= 2)
        return 0;

    fdDir = opendir("/dev/");
    if (!fdDir) {
        return -1;
    }

    while ((dire = readdir(fdDir)) != NULL) {
        // stats
        snprintf(szTemp1, sizeof(szTemp1), "/dev/%s", dire->d_name);
        nRes = lstat(szTemp1, &st);
      
        // if this is a sym link
        if ((nRes == 0) && S_ISLNK(st.st_mode)) {
            memset(szTemp, 0, sizeof(szTemp));
            nRes = readlink(szTemp1, szTemp, sizeof(szTemp));
      
            memset(szTemp2, 0, sizeof(szTemp2));
            snprintf(szTemp2, sizeof(szTemp2), "/dev/%s", szTemp);
      
            if (strcmp(szDevfs, szTemp2) == 0) {
                memset(szShort, 0, nMaxShort);
                snprintf(szShort, nMaxShort, "%s", szTemp1);
                closedir(fdDir);
                return 0;
            }
        }
    }

    closedir(fdDir);

    return 0;
}

void QP_LibParted::get_filesystem(QP_FileSystem *filesystem) {
    /*---scan all filesystem supported by parted---*/
    PedFileSystemType* walk = NULL;
    for (walk = ped_file_system_type_get_next (NULL); walk;
         walk = ped_file_system_type_get_next (walk)) {

        /*---if the filesystem is supported by parted add it to the list---*/
        filesystem->addFileSystem(walk->name,
                                  walk->ops->create,
                                  walk->ops->resize,
                                  walk->ops->copy,   //Move and copy is the same thing for parted!
                                  walk->ops->copy);

    }
        
    /*---loop into fswraplist, and add it to sigTimer ('cause wrapper handle by itself)---*/
    QP_FSWrap *p;
    for (p = (QP_FSWrap *)filesystem->fswraplist.first(); p;
            p = (QP_FSWrap *)filesystem->fswraplist.next()) {
        connect(p, SIGNAL(sigTimer(int, QString, QString)),
                this, SIGNAL(sigTimer(int, QString, QString)));
    }
}

void QP_LibParted::partition_get_flags(QP_PartInfo *partinfo, PedPartition *part) {
    if (ped_partition_is_flag_available(part, PED_PARTITION_BOOT)) {
        partinfo->_canBeActive = true;
        bool active = bool(ped_partition_get_flag(part, PED_PARTITION_BOOT));
        partinfo->_active = active;
        if (active) _partActive = partinfo;
    }
}

bool QP_LibParted::partition_set_flag_active(QP_PartInfo *partinfo, bool active) {
    PedDisk *disk;
    PedPartition *part;

    disk = ped_disk_new(dev);
    if (!disk) {
        _message = QString(ERROR_PED_DISK_NEW);
        goto error;
    }

	part = ped_disk_get_partition(disk, partinfo->num);
    if (!part) {
        _message = QString(ERROR_PED_DISK_GET_PARTITION);
        goto error_destroy_disk;
    }

    if (!ped_partition_is_flag_available(part, PED_PARTITION_BOOT)) {
        _message = QString(tr("Cannot change the active status on this partition"));
        goto error_destroy_disk;
        return false;
    }

    if (!ped_partition_set_flag(part, PED_PARTITION_BOOT, active)) {
        _message = QString(ERROR_PED_PARTITION_SET_FLAG);
        goto error_destroy_disk;
        return false;
    }

    ped_disk_commit(disk);
    ped_disk_destroy(disk);
    return true;

error_destroy_disk:
    ped_disk_destroy(disk);
error:
    return false;
}

QP_PartInfo * QP_LibParted::partActive() {
    return _partActive;
}

PedSector QP_LibParted::get_left_bound (PedSector sector, PedDisk* disk) {
	PedPartition*	under_sector;

	under_sector = ped_disk_get_partition_by_sector (disk, sector);
	PED_ASSERT (under_sector != NULL, return sector);

	if (under_sector->type & PED_PARTITION_FREESPACE)
		return under_sector->geom.start;
	else
		return sector;
}

PedSector QP_LibParted::get_right_bound (PedSector sector, PedDisk* disk) {
	PedPartition*	under_sector;

	under_sector = ped_disk_get_partition_by_sector (disk, sector);
	PED_ASSERT (under_sector != NULL, return sector);

	if (under_sector->type & PED_PARTITION_FREESPACE)
		return under_sector->geom.end;
	else
		return sector;
}

/* if the gap between the new partition and the partition-to-be-aligned is
 * less than MIN_FREESPACE, then gobble up the gap!
 */
int QP_LibParted::grow_over_small_freespace (PedGeometry* geom, PedDisk* disk) {
	PedSector	start;
	PedSector	end;

	/* hack: give full control for small partitions */
	if (geom->length < MIN_FREESPACE * 5)
		return 1;

	start = get_left_bound (geom->start, disk);
	PED_ASSERT (start < geom->end, return 0);
	if (geom->start - start < MIN_FREESPACE)
		ped_geometry_set_start (geom, start);

	end = get_right_bound (geom->end, disk);
	PED_ASSERT (end > geom->start, return 0);
	if (end - geom->end < MIN_FREESPACE)
		ped_geometry_set_end (geom, end);
	return 1;
}

bool QP_LibParted::set_system(QP_PartInfo *partinfo, QP_FileSystemSpec *fsspec) {
    PedDisk *disk;
    PedPartition *part;
    const PedFileSystemType *fs_type = NULL;

    if (fsspec == NULL) {
        _message = QString(tr("A bug was found in QTParted during set_system, please report it!"));
        return false;
    }

    disk = ped_disk_new(dev);
    if (!disk) {
        _message = QString(ERROR_PED_DISK_NEW);
        goto error;
    }

	part = ped_disk_get_partition(disk, partinfo->num);
    if (!part) {
        _message = QString(ERROR_PED_DISK_GET_PARTITION);
        goto error_destroy_disk;
    }

    fs_type = ped_file_system_type_get(fsspec->name().latin1());
    if (!fs_type) {
        _message = QString(tr("A bug was found in QTParted during ped_file_system_type_get, please report it!"));
        goto error_destroy_disk;
    }

	if (!ped_partition_set_system(part, fs_type)) {
        _message = QString(ERROR_PED_PARTITION_SET_SYSTEM);
		goto error_destroy_disk;
    }

	if (!ped_disk_commit(disk))
		goto error_destroy_disk;
    
    ped_disk_destroy(disk);
    return true;

error_destroy_disk:
    ped_disk_destroy(disk);
error:
    return false;
}

int QP_LibParted::mkpart(PedSector start, PedSector end, PedPartitionType part_type, QP_FSWrap *fswrap, QString label) {
    PedDisk *disk;
    PedPartition *part;
    PedFileSystemType *fs_type = NULL;
    PedConstraint *constraint;
    QString devstr;

    _message = QString::null;

	disk = ped_disk_new(dev);
	if (!disk) {
        _message = QString(ERROR_PED_DISK_NEW);
		goto error;
    }
	constraint = ped_constraint_any(dev);
	if (!constraint)
		goto error_destroy_disk;

    /*---if it exist a wrapper change the file_system_id---*/
    if (fswrap) fs_type = ped_file_system_type_get(fswrap->fsname());

	part = ped_partition_new (disk, part_type, fs_type, start, end);
	if (!part) {
        _message = QString(ERROR_PED_PARTITION_NEW);
		goto error_destroy_constraint;
    }
    
	if (!grow_over_small_freespace (&part->geom, disk))
		goto error_destroy_part;
    
	if (!ped_disk_add_partition (disk, part, constraint)) {
        _message = QString(ERROR_PED_DISK_ADD_PARTITION);
		goto error_destroy_part;
    }

    /*---get the device name made.
     *   with devfs it is /dev/ide/tricktrack/etc/etc/partNumber
     *   if there is not devfs it is /dev/hd*number
     */
    if (flagDevfsEnabled) {
        QString devpath = QString(dev->path);
        devpath.truncate(strlen(dev->path)-4); //remove ending "disc" characters
        devpath.append("part");
        devstr = QString("%1%2")
                        .arg(devpath)
                        .arg(part->num);
    } else {
        devstr = QString("%1%2")
                        .arg(dev->path)
                        .arg(part->num);
    }
    
	ped_constraint_destroy (constraint);

    /*TODO*/
/*	if (!_solution_check_distant (start, end,
				      part->geom.start, part->geom.end,
		_("You requested to create a partition at %.3f-%.3fMb. The "
		  "closest Parted can manage is %.3f-%.3fMb.")))
		goto error_remove_part;*/

	if (!ped_partition_set_system(part, fs_type)) {
        _message = QString(ERROR_PED_PARTITION_SET_SYSTEM);
		goto error_destroy_disk;
    }

	if (!ped_disk_commit(disk))
		goto error_destroy_disk;
	ped_disk_destroy (disk);

    /*---if it exist a wrapper just make the filesystem---*/
    if (fswrap) {
        bool rc = fswrap->mkpartfs(devstr, label);
        if (!rc) _message = fswrap->message();
        return rc;
    }

	return true;

/*error_remove_part:*/
	ped_disk_remove_partition (disk, part);
error_destroy_part:
	ped_partition_destroy (part);
error_destroy_constraint:
	ped_constraint_destroy (constraint);
error_destroy_disk:
	ped_disk_destroy (disk);
error:
	return false;
}

int QP_LibParted::mkpartfs(QTParted::Type type,
                           QP_FileSystemSpec *fsspec,
                           PedSector start,
                           PedSector end,
                           QString label) {

    _message = QString::null;

    /*---want to make an extended partition? Than call mkpart!---*/
    if (type == QTParted::extended) {
        bool rc = mkpart(start, end, PED_PARTITION_EXTENDED, NULL, QString::null);
        if (!rc) emitSigTimer(100, message(), QString::null);
        else     emitSigTimer(100, SUCCESS, QString::null);
        return rc;
    }

    /*---it is not possible that part_type is different than primary and logical... 'cause
     *---the only case is when a partition is extended, but just above i tested if it was!---*/
    PedPartitionType part_type;
    if (type == QTParted::primary)
        part_type = (PedPartitionType)0;
    else if (type == QTParted::logical)
        part_type = PED_PARTITION_LOGICAL;
    else {
        _message = QString("A bug was found in QTParted during mkpartfs, please report it!");
        return false;
    }

    /*---if it exist a wrapper maybe libparted doesn't support this filesystem...---*/
    if (fsspec->fswrap()) {
        bool rc = false;
        /*---the wrapper support the mkpartfs?---*/
        if (fsspec->fswrap()->wrap_create) {
            rc = mkpart(start, end, part_type, fsspec->fswrap(), label);
            if (!rc) emitSigTimer(100, message(), QString::null);
            else     emitSigTimer(100, SUCCESS, QString::null);
            return rc;
        }
    }
    
	PedPartition *part;
	const PedFileSystemType *fs_type;
	PedConstraint *constraint;
	PedFileSystem *fs;

	PedDisk *disk = ped_disk_new (dev);
	if (!disk) {
        _message = QString(ERROR_PED_DISK_NEW);
		goto error;
    }

	fs_type = ped_file_system_type_get (fsspec->name().latin1());
	if (!fs_type)
        goto error_destroy_disk;
	constraint = ped_file_system_get_create_constraint (fs_type, dev);

	part = ped_partition_new (disk, part_type, fs_type, (int)start, (int)end);
	if (!part) {
        _message = QString(ERROR_PED_PARTITION_NEW);
		goto error_destroy_constraint;
    }

	if (!grow_over_small_freespace (&part->geom, disk))
		goto error_destroy_part;

	if (!ped_disk_add_partition (disk, part, constraint)) {
        _message = QString(ERROR_PED_DISK_ADD_PARTITION);
		goto error_destroy_part;
    }

    /* TODO */
/*	if (!_solution_check_distant (start, end,
				      part->geom.start, part->geom.end,
		_("You requested to create a partition at %.3f-%.3fMb. The "
		  "closest Parted can manage is %.3f-%.3fMb.")))
		goto error_remove_part;*/

	fs = ped_file_system_create(&part->geom, fs_type, timer); //NULL); //TODO timer);
	if (!fs) 
		goto error_remove_part;
	ped_file_system_close(fs);
	ped_constraint_destroy(constraint);

	if (!ped_partition_set_system(part, fs_type))
		goto error_destroy_disk;

	if (!ped_disk_commit(disk))
		goto error_destroy_disk;
	ped_disk_destroy (disk);

    emitSigTimer(100, SUCCESS, QString::null);
	return true;

error_remove_part:
	ped_disk_remove_partition (disk, part);
error_destroy_part:
	ped_partition_destroy (part);
error_destroy_constraint:
	ped_constraint_destroy (constraint);
error_destroy_disk:
	ped_disk_destroy (disk);
error:
    emitSigTimer(100, message(), QString::null);
	return false;
}

int QP_LibParted::rm(int num) {
    PedDisk *disk;
    PedPartition *part;

    disk = ped_disk_new(dev);
    if (!disk) {
        _message = QString(ERROR_PED_DISK_NEW);
        goto error;
    }

	part = ped_disk_get_partition(disk, num);
    if (!part) {
        _message = QString(ERROR_PED_DISK_GET_PARTITION);
        goto error_destroy_disk;
    }

	if (ped_partition_is_busy(part))
        goto error_destroy_disk;

    ped_disk_delete_partition(disk, part);
    ped_disk_commit(disk);
    ped_disk_destroy(disk);
    return true;

error_destroy_disk:
    ped_disk_destroy(disk);
error:
    return false;
}

bool QP_LibParted::partition_is_busy(int num) {
    PedDisk *disk;
    PedPartition *part;
    _message = QString::null;

    /*---get the PedDisk from the device---*/
    disk = ped_disk_new(dev);
    if (!disk) {
        _message = QString(ERROR_PED_DISK_NEW);
        goto error;
    }

    /*---get the partition structure---*/
    part = ped_disk_get_partition(disk, num);
    if (!part) {
        _message = QString(ERROR_PED_DISK_GET_PARTITION);
        goto error_destroy_disk;
    }

    /*---test if the partition is busy (ie mounted)---*/
    if (ped_partition_is_busy(part)) {
        _message = QString(tr("The partition is mounted."));
        goto error_destroy_disk;
    }

    return false;
    
error_destroy_disk:
    ped_disk_destroy(disk);
error:
    return true;
}

bool QP_LibParted::move(QP_PartInfo *partinfo, PedSector start, PedSector end) {
    PedDisk *disk;
    PedPartition *part;
    PedGeometry old_geom;
    PedGeometry new_geom;
    PedFileSystem *fs;
    PedFileSystem *fs_copy;
    PedConstraint *constraint;

    /*---libparted doesn't implement move of extended partition---*/
    if (partinfo->type == QTParted::extended) {
        _message = QString(tr("Can't move extended partitions."));
        goto error;
    }
    
    disk = ped_disk_new(dev);
    if (!disk) {
        _message = QString(ERROR_PED_DISK_NEW);
        goto error;
    }
    
    /*---get the partition info---*/
    part = ped_disk_get_partition(disk, partinfo->num);
    if (!part) {
        _message = QString(ERROR_PED_DISK_GET_PARTITION);
        goto error_destroy_disk;
    }

    /*---if a partition is used...---*/
    if (!_partition_warn_busy(part))
        goto error_destroy_disk;

    old_geom = part->geom;

    /* open fs, and get copy constraint */
    fs = ped_file_system_open (&old_geom);
    if (!fs)
        goto error_destroy_disk;
    constraint = ped_file_system_get_copy_constraint (fs, dev);

    /* set / test on "disk" */
    if (!ped_geometry_init (&new_geom, dev, start, end - start + 1))
        goto error_destroy_constraint;

    /*TODO 
    if (!_grow_over_small_freespace (&new_geom, disk))
        goto error_destroy_constraint;*/

    if (!ped_disk_set_partition_geom(disk, part, constraint, new_geom.start, new_geom.end))
        goto error_destroy_constraint;
	ped_constraint_destroy (constraint);
    
    if (ped_geometry_test_overlap (&old_geom, &part->geom)) {
        _message = QString(tr("Can't move a partition onto itself. Try using resize, perhaps?"));
        goto error_close_fs;
    }

    /*TODO
	if (!_solution_check_distant (start, end, part->geom.start, part->geom.end,
		_("You requested to move the partition to %.3f-%.3fMb. The "
		  "closest Parted can manage is %.3f-%.3fMb.")))
		goto error_close_fs;
    */


/* do the move */
	fs_copy = ped_file_system_copy(fs, &part->geom, timer);
	if (!fs_copy)
		goto error_close_fs;
	ped_file_system_close(fs_copy);
	ped_file_system_close(fs);
	if (!ped_disk_commit(disk))
		goto error_destroy_disk;
	ped_disk_destroy (disk);
    return true;

error_destroy_constraint:
    printf("error_destroy_constraint\n");
	ped_constraint_destroy(constraint);
error_close_fs:
    printf("error_close_fs\n");
	ped_file_system_close(fs);
error_destroy_disk:
    printf("error_destroy_disk\n");
	ped_disk_destroy(disk);
error:
    printf("error... e basta!\n");
    return false;
}

bool QP_LibParted::resize(QP_PartInfo *partinfo, PedSector start, PedSector end) {
    _message = QString::null;

    PedDisk *disk;
    PedPartition *part;
    PedFileSystem *fs;
    PedConstraint *constraint;
    PedGeometry new_geom;

    disk = ped_disk_new (dev);
    if (!disk) {
        _message = QString(ERROR_PED_DISK_NEW);
        goto error;
    }

    /*---get the partition info---*/
    part = ped_disk_get_partition(disk, partinfo->num);
    if (!part) {
        _message = QString(ERROR_PED_DISK_GET_PARTITION);
        goto error_destroy_disk;
    }

    /*---if a partition is used...---*/
    if (!_partition_warn_busy(part))
        goto error_destroy_disk;

    if (!ped_geometry_init (&new_geom, dev, start, end - start + 1)) {
        _message = QString(tr("An error happen during ped_geometry_init call."));
        goto error_destroy_disk;
    }
    /*
     * TODO
    if (!_grow_over_small_freespace (&new_geom, disk))
        goto error_destroy_disk;*/

    if (part->type == PED_PARTITION_EXTENDED) {
        constraint = ped_constraint_any(dev);
        if (!ped_disk_set_partition_geom(disk, part, constraint, new_geom.start, new_geom.end)) {
            _message = QString(tr("An error happen during ped_disk_set_partition_geom call."));
            goto error_destroy_constraint;
        }
        //TODO
        /*if (!_solution_check_distant (start, end,
                          part->geom.start, part->geom.end,
            _("You requested to resize the partition to "
              "%.3f-%.3fMb. The closest Parted can manage is "
              "%.3f-%.3fMb.")))
            goto error_destroy_constraint;*/
        ped_partition_set_system(part, NULL);
    } else {
        fs = ped_file_system_open(&part->geom);
        if (!fs) {
            _message = QString(tr("An error happen during ped_file_system_open call."));
            goto error_destroy_disk;
        }
        constraint = ped_file_system_get_resize_constraint (fs);
        if (!ped_disk_set_partition_geom (disk, part, constraint,
                          new_geom.start, new_geom.end)) {
            _message = QString(tr("An error happen during ped_disk_set_partition_geom call."));
            goto error_close_fs;
        }
        //TODO
        /*if (!_solution_check_distant (start, end,
                          part->geom.start, part->geom.end,
            _("You requested to resize the partition to "
              "%.3f-%.3fMb. The closest Parted can manage is "
              "%.3f-%.3fMb.")))
            goto error_close_fs;*/
        if (!ped_file_system_resize(fs, &part->geom, timer)) {
            _message = QString(tr("An error happen during ped_file_system_resize call."));
            goto error_close_fs;
        }
        /* may have changed... eg fat16 -> fat32 */
        ped_partition_set_system(part, fs->type);
        ped_file_system_close(fs);
    }

    ped_disk_commit(disk);
    ped_constraint_destroy(constraint);
    ped_disk_destroy(disk);
    return true;

error_close_fs:
	ped_file_system_close (fs);
error_destroy_constraint:
	ped_constraint_destroy (constraint);
error_destroy_disk:
	ped_disk_destroy (disk);
error:
	return false;
}

bool QP_LibParted::set_geometry(QP_PartInfo *partinfo, PedSector start, PedSector end) {
    _message = QString::null;

    PedDisk *disk;
    PedPartition *part;
    PedConstraint *constraint;
    PedGeometry new_geom;

    disk = ped_disk_new (dev);
    if (!disk) {
        _message = QString(ERROR_PED_DISK_NEW);
        goto error;
    }

    /*---get the partition info---*/
    part = ped_disk_get_partition(disk, partinfo->num);
    if (!part) {
        _message = QString(ERROR_PED_DISK_GET_PARTITION);
        goto error_destroy_disk;
    }

    if (!_partition_warn_busy(part))
        goto error_destroy_disk;

    if (!ped_geometry_init (&new_geom, dev, start, end - start + 1)) {
        _message = QString(tr("An error happen during ped_geometry_init call."));
        goto error_destroy_disk;
    }
    /*
     * TODO
    if (!_grow_over_small_freespace (&new_geom, disk))
        goto error_destroy_disk;*/

    constraint = ped_constraint_any(dev);
    if (!ped_disk_set_partition_geom(disk, part, constraint, new_geom.start, new_geom.end)) {
        _message = QString(tr("An error happen during ped_disk_set_partition_geom call."));
        goto error_destroy_constraint;
    }
    //TODO
    /*if (!_solution_check_distant (start, end,
                      part->geom.start, part->geom.end,
        _("You requested to resize the partition to "
          "%.3f-%.3fMb. The closest Parted can manage is "
          "%.3f-%.3fMb.")))
        goto error_destroy_constraint;*/

    ped_disk_commit(disk);
    ped_constraint_destroy(constraint);
    ped_disk_destroy(disk);
    return true;

error_destroy_constraint:
	ped_constraint_destroy (constraint);
error_destroy_disk:
	ped_disk_destroy (disk);
error:
	return false;
}

QString QP_LibParted::message() {
    return _message;
}

void QP_LibParted::emitSigTimer(int percent, QString state, QString timer) {
    emit sigTimer(percent, state, timer);
}

float QP_LibParted::mb_hdsize() {
    return _mb_hdsize;
}

void QP_LibParted::setFastScan(bool FastScan) {
    _FastScan = FastScan;
}

bool QP_LibParted::get_partfilesystem_info(PedPartition *part, QP_PartInfo *partinfo) {
    /*---get the label of the partition---*/
    const char *label = ped_partition_get_name(part);
    if (label != NULL)
        partinfo->label = label;

    /*---does it exist a filesystem wrapper?---*/
    if (partinfo->fsspec->fswrap()) {
        if (partinfo->fsspec->fswrap()->wrap_min_size) {
            PedSector min_size = partinfo->fsspec->fswrap()->min_size(partinfo->partname());
            if (min_size != -1)
                partinfo->min_size = min_size;
        }

        return true;
    }
    
    
    /*---is the filesystem supported by parted?---*/
    PedFileSystem *fs = ped_file_system_open (&part->geom);
    if (!fs) return true;

    /*---get the minimum filesystem size---*/
    PedConstraint *resize_constraint = ped_file_system_get_resize_constraint(fs);
    if (resize_constraint) {
        partinfo->min_size = resize_constraint->min_size;
        ped_constraint_destroy (resize_constraint);
    }

    ped_file_system_close(fs);

    return true;
}

/*---code bring from parted.c---*/
/*---return if a partition is used!---*/
bool QP_LibParted::_partition_warn_busy(PedPartition* part) {
    char *path = ped_partition_get_path(part);

    if (ped_partition_is_busy(part)) {
        QString label = QString(tr("Partition %s is being used."));
        if (ped_exception_throw(PED_EXCEPTION_ERROR,
                                (PedExceptionOption)PED_EXCEPTION_IGNORE_CANCEL,
                                label.latin1(),
                                path) != PED_EXCEPTION_IGNORE) goto error_free_path;
    }
    ped_free(path);
    return true;

error_free_path:
    ped_free(path);

//error:
    return false;
}
/*-end of QP_LibParted---------------------------------------------------------------------------*/
