/*
    qtparted - a frontend to libparted for manipulating disk partitions
    Copyright (C) 2002-2003 Vanni Brutto

    Vanni Brutto <zanac (-at-) libero dot it>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <qapplication.h>
#include <qsplitter.h>
#include <qlabel.h>
#include <qhbox.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qradiobutton.h>
#include <qstatusbar.h>
#include <qtoolbar.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qmessagebox.h>
#include "qp_common.h"
#include "qp_window.h"
#include "qp_filesystem.h"
#include "qp_dlgabout.h"
#include "qp_options.h"

#include "xpm/tool_property.xpm"
#include "xpm/tool_delete.xpm"
#include "xpm/tool_resize.xpm"
#include "xpm/tool_move.xpm"
#include "xpm/tool_quit.xpm"
#include "xpm/tool_new.xpm"
#include "xpm/tool_whatthis.xpm"
#include "../data/qtparted.xpm"



QP_MainWindow::QP_MainWindow(QP_Settings *qpsettings, QWidget *parent, const char *name)
                  :QMainWindow(parent, name) {
    setCaption(QString("%1 v%2").arg(PROG_NAME).arg(VERSION));

    /*---load the setting from disk---*/
    settings = qpsettings;

    createAction();
    addToolBar();
    addMenuBar();
    addStatusBar();

    /*---init "enabled" status on menuitem---*/
    InitMenu();

    /*---create the dialog used for "create" new partition---*/
    dlgcreate = new QP_dlgCreate(this, "dlgCreate");

    /*---create the dialog used for "resize" partition---*/
    dlgresize = new QP_dlgResize(this, "dlgResize");

    /*---create the dialog used for "progress"---*/
    dlgprogress = new QP_dlgProgress(this, "dlgProgress");

    /*---create the dialog used for "configuration"---*/
    dlgconfig = new QP_dlgConfig(this, "dlgConfig");

    /*---this is the central widget of the window (where i will attach the qsplitter---*/
    central = new QVBox(this);
    setCentralWidget(central);



    /*---the window is so designed:
     *
     * .------------------------------.
     * |QSplitter     |               |
     * |              |               |
     * | +----------+ | +-----------+ |
     * | |NavView   | | |DiskView   | |
     * | |          | | |           | |
     * | |          | | |           | |
     * | +----------+ | +-----------+ |
     * |              |               |
     * `------------------------------'
     */

    
    /*---make the vertical splitter---*/
    QSplitter *navSplit = new QSplitter(central);
    navSplit->setOpaqueResize(true); //I like opaque resize ;-)

    /*---add the navigator view (on the left) to the splitter---*/
    navview = new QP_NavView(navSplit);
    /*---i like that navview will not be always resized!---*/
    navSplit->setResizeMode(navview, QSplitter::KeepSize);
    /*---connect the selected signal (when user, for example, select /dev/hda)---*/
    connect(navview, SIGNAL(selected(QString)),
            this, SLOT(slotSelected(QString)));

    /*---add the DiskView widget---*/
    diskview = new QP_DiskView(navSplit);
    /*---emitted when you select a partition---*/
    connect(diskview, SIGNAL(sigSelectPart(QP_PartInfo *)),
            this, SLOT(slotSelectPart(QP_PartInfo *)));
    /*---emit when the user want to popup the context menu---*/
    connect(diskview, SIGNAL(sigPopup(QPoint)),
            this, SLOT(slotPopup(QPoint)));
    /*---connect the sigTimer used for dlgprogress---*/
    connect(diskview, SIGNAL(sigTimer(int, QString, QString)),
            dlgprogress, SLOT(slotTimer(int, QString, QString)));
}     


QP_MainWindow::~QP_MainWindow() {
}

void QP_MainWindow::init() {
    emit sigSplashInfo(tr("Getting devices"));
    navview->init();

    /*---prevent from memory leak: when list are cleared destroy QAction object!---*/
    lstSetSystem.setAutoDelete(true);

    /*---now populate the disks menu---*/
    buildDisksMenu();

    /*---load user settings---*/
    emit sigSplashInfo(tr("load user settings"));
    loadSettings();

    emit sigSplashInfo(tr("Ready"));
}

void QP_MainWindow::refreshDiskView() {
    /*---show a progress dialog for long operation---*/
    InitProgressDialog();

    /*---refresh diskview widget!---*/
    diskview->refresh();

    /*---destroy the progress dialog---*/
//    DoneProgressDialog();
    
    /*---closethe progress dialog---*/
    dlgprogress->hide();
}

void QP_MainWindow::setpopupmenu(QPopupMenu *popupmenu) {
    _popupmenu = popupmenu;
}

QPopupMenu* QP_MainWindow::popupmenu() {
    return _popupmenu;
}

void QP_MainWindow::createAction() {
    /*---Quit button (used in File menu)---*/
    actQuit = new QAction(this, "Quit button");
    actQuit->setText(tr("&Quit"));
    actQuit->setIconSet(QPixmap(tool_quit));
    actQuit->setToolTip(tr("Quit"));
    actQuit->setWhatsThis(tr("Quit from the application"));
    actQuit->setEnabled(true);
    connect(actQuit, SIGNAL(activated()),
            qApp, SLOT(quit()));

    /*---Property button (used in operations menu)---*/
    actProperty = new QAction(this, "property button");
    actProperty->setText(tr("&Property"));
    actProperty->setIconSet(QPixmap(tool_property));
    actProperty->setToolTip(tr("Property"));
    actProperty->setWhatsThis(tr("Property of the selected partition"));
    actProperty->setEnabled(false);
    connect(actProperty, SIGNAL(activated()),
            this, SLOT(slotProperty()));

    /*---Create button (used in operations menu)---*/
    actCreate = new QAction(this, "create button");
    actCreate->setText(tr("&Create"));
    actCreate->setIconSet(QPixmap(tool_new));
    actCreate->setToolTip(tr("Create"));
    actCreate->setWhatsThis(tr("Create a new partition"));
    actCreate->setEnabled(false);
    connect(actCreate, SIGNAL(activated()),
            this, SLOT(slotCreate()));

    /*---Resize button (used in operations menu)---*/
    actResize = new QAction(this, "resize button");
    actResize->setText(tr("&Resize"));
    actResize->setIconSet(QPixmap(tool_resize));
    actResize->setToolTip(tr("Resize"));
    actResize->setWhatsThis(tr("Resize a partition"));
    actResize->setEnabled(false);
    connect(actResize, SIGNAL(activated()),
            this, SLOT(slotResize()));

    /*---Move button (used in operations menu)---*/
    actMove = new QAction(this, "move button");
    actMove->setText(tr("&Move"));
    actMove->setIconSet(QPixmap(tool_move));
    actMove->setToolTip(tr("Move"));
    actMove->setWhatsThis(tr("Move a partition"));
    actMove->setEnabled(false);
    connect(actMove, SIGNAL(activated()),
            this, SLOT(slotMove()));

    /*---Delete button (used in operations menu)---*/
    actDelete = new QAction(this, "Delete button");
    actDelete->setText(tr("&Delete"));
    actDelete->setIconSet(QPixmap(tool_delete));
    actDelete->setToolTip(tr("Delete"));
    actDelete->setWhatsThis(tr("Delete a new partition"));
    actDelete->setEnabled(false);
    connect(actDelete, SIGNAL(activated()),
            this, SLOT(slotDelete()));

    /*---Config button (used in options menu)---*/
    actConfig = new QAction(this, "Config button");
    actConfig->setText(tr("&Configuration..."));
    actConfig->setIconSet(QPixmap(tool_property));
    actConfig->setToolTip(tr("Configuration of QTParted"));
    actConfig->setWhatsThis(tr("Open the configuration dialog"));
    actConfig->setEnabled(true);
    connect(actConfig, SIGNAL(activated()),
            this, SLOT(slotConfig()));
   
    /*---What this button (used in toolbutton bar)---*/
    actWhatThis = new QAction(this, "Whatsthis button");
    actWhatThis->setText(tr("What's &This"));
    actWhatThis->setIconSet(QPixmap(tool_whatthis));
    actWhatThis->setToolTip(tr("What's This"));
    actWhatThis->setWhatsThis(tr("Enter <b>What's This</b> mode"
                              "It enables the user to ask for help "
                              "about widgets on the screen."));
    actWhatThis->setEnabled(true);
    actWhatThis->setAccel(SHIFT+Key_F1);
    connect(actWhatThis, SIGNAL(activated()),
            this, SLOT(slotWhatsThis()));

    /*---About button (used in operations help)---*/
    actAbout = new QAction(this, "About QTParted");
    actAbout->setText(tr("About &QTParted"));
    actAbout->setToolTip(tr("About QTParted"));
    actAbout->setWhatsThis(tr("Information about QTParted"));
    connect(actAbout, SIGNAL(activated()),
            this, SLOT(slotAbout()));

    /*---About QT button (used in operations help)---*/
    actAboutQT = new QAction(this, "About QT Button");
    actAboutQT->setText(tr("About Q&T"));
    actAboutQT->setToolTip(tr("About QT"));
    actAboutQT->setWhatsThis(tr("Information about QT"));
    connect(actAboutQT, SIGNAL(activated()),
            this, SLOT(slotAboutQT()));
}

void QP_MainWindow::addMenuBar() {
    /*---File menu---*/
    QPopupMenu *mnuFile = new QPopupMenu;
    menuBar()->insertItem(tr("&File"), mnuFile);
    actQuit->addTo(mnuFile);
   

    /*---Action menu---*/
    mnuOperations = new QPopupMenu;
    mnuOperations->setCheckable(true);
    menuBar()->insertItem(tr("&Operations"), mnuOperations);
    actProperty->addTo(mnuOperations);
    actCreate->addTo(mnuOperations);
    actResize->addTo(mnuOperations);
    actMove->addTo(mnuOperations);
    actDelete->addTo(mnuOperations);
    //
    mnuOperations->insertSeparator();
    //
    mnuSetActiveID = mnuOperations->insertItem(tr("Set Active..."));
    mnuOperations->connectItem(mnuSetActiveID, this, SLOT(slotSetActive()));
    mnuOperations->setItemEnabled(mnuSetActiveID, false);


    /*---add "set system id"---*/
    mnuSetSystem = new QPopupMenu;
    //FIXME: adding this line a new menu will appare... but it is buggy for now :(
#ifdef UNSTABLE
    mnuOperations->insertItem(tr("&Change System ID"), mnuSetSystem);
#endif
    mnuSetSystem->setEnabled(false);

    /*---set popupmenu for the action menu!---*/
    setpopupmenu(mnuOperations);


    /*---Disk menu---*/
    mnuDisks = new QPopupMenu;
    menuBar()->insertItem(tr("&Disks"), mnuDisks);


    /*---Options menu---*/
    QPopupMenu *mnuOptions = new QPopupMenu;
    menuBar()->insertItem(tr("&Options"), mnuOptions);
    actConfig->addTo(mnuOptions);

   
    /*---Help menu---*/
    QPopupMenu *mnuHelp = new QPopupMenu;
    menuBar()->insertItem(tr("&Help"), mnuHelp);
    actWhatThis->addTo(mnuHelp);
    mnuHelp->insertSeparator();
    actAbout->addTo(mnuHelp);
    actAboutQT->addTo(mnuHelp);
}

void QP_MainWindow::addStatusBar() {
    /*---This functions is dummy! i have not yet decided if a statusbar is useful!---*/

    /*---make an hbox and add it in the statusbar---*/
    QHBox *hbox = new QHBox(statusBar());
    hbox->setMargin(5);
    hbox->setSpacing(6);

    /*---make a label, size it, set the text... and attach it to the hbox---*/
    QLabel *lbl1 = new QLabel(hbox, "message");
    lbl1->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    lbl1->setAlignment(AlignLeft);
    QFont boldfont; boldfont.setWeight(QFont::Bold);
    lbl1->setFont(boldfont);
    lbl1->setText("Message");
    lbl1->setMinimumHeight(lbl1->sizeHint().height());
    lbl1->setText("QTParted :)");
    QToolTip::add(lbl1, "Message area");
    
    /*---add a separator in the statusbar---*/
    QFrame *frame = new QFrame(hbox, "frame");
    frame->setFrameShadow(QFrame::Sunken);
    frame->setFrameShape(QFrame::VLine);

    /*---make a label, size it, set the text... and attach it to the hbox---*/
    QLabel *lbl2 = new QLabel(hbox, "message");
    lbl2->setSizePolicy(QSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed));
    lbl2->setAlignment(AlignLeft);
    boldfont.setWeight(QFont::Bold);
    lbl2->setFont(boldfont);
    lbl2->setText("Message");
    lbl2->setMinimumHeight(lbl2->sizeHint().height());
    lbl2->setText("by Zanac (c) 2002-2003");
    QToolTip::add(lbl2, "Message area");
    
    frame = new QFrame(hbox, "frame");
    frame->setFrameShadow(QFrame::Sunken);
    frame->setFrameShape(QFrame::VLine);

    QLabel *lblmsg = new QLabel(hbox, "message");
    lblmsg->setAlignment(AlignLeft);
    boldfont.setWeight(QFont::Bold);
    lblmsg->setFont(boldfont);
    lblmsg->setText("Message");
    lblmsg->setMinimumHeight(lblmsg->sizeHint().height());
    lblmsg->setText(QString::null);
    QToolTip::add(lblmsg, "Message area");

    /*---add the hbox to the statusbar---*/
    statusBar()->addWidget(hbox, 1);

    /*---add the "grip" on the right of the statusbar---*/
    statusBar()->setSizeGripEnabled(true);
}

void QP_MainWindow::addToolBar() {
    /*---Operations toolbar---*/
    QToolBar *toolOperations = new QToolBar(this, "toolOperations");
    actProperty->addTo(toolOperations);
    actCreate->addTo(toolOperations);
    actResize->addTo(toolOperations);
    actMove->addTo(toolOperations);
    actDelete->addTo(toolOperations);

    /*---What's this toolbar---*/
    QToolBar *toolWhatThis = new QToolBar(this, "toolWhatThis");
    actWhatThis->addTo(toolWhatThis);
}

void QP_MainWindow::buildDisksMenu() {
    navview->agDevices()->addTo(mnuDisks);
}

void QP_MainWindow::loadSettings() {
    /*---set the layout---*/
    diskview->setLayout(settings->layout());
}

void QP_MainWindow::slotCreate() {
    /*---init of the dialog box used for create new partition---*/
    dlgcreate->init_dialog();

    /*---loop into filesystemlist, and add it to combobox if you it has create flag---*/
    QP_FileSystemSpec *p;
    for (p = (QP_FileSystemSpec*)diskview->filesystem->filesystemlist.first(); 
            p; p = (QP_FileSystemSpec*)diskview->filesystem->filesystemlist.next())
        if (p->create())
            dlgcreate->addFileSystem(p->name());

    /*---the "addType" method of dlgcreate add new type to the combobox---*/
    
    /*---the user want to create a primary/extended partition---*/
    if (diskview->selPartInfo()->type == QTParted::primary) {
        dlgcreate->addTypePrimary();

        /*---'cause there isn't any extended partition add it in the combo box---*/
        if (!diskview->libparted->has_extended) 
            dlgcreate->addTypeExtended();
    } else
        /*---else... add a logical!---*/
        dlgcreate->addTypeLogical();

    //old! float maxsize = diskview->selPartInfo()->mb_end() - diskview->selPartInfo()->mb_start();
    /*---get the maxsize of the partition selected. This is the max PedSector size that u can create---*/
    PedSector maxsize = diskview->selPartInfo()->end - diskview->selPartInfo()->start;
    dlgcreate->setMaxSize(maxsize);

    /*---show dialog, and get return code (useful for getting "accepted"/"cancel" of dialog---*/
    int code = dlgcreate->show_dialog();
    if (code == QDialog::Accepted) {
        PedSector start;
        PedSector end;

        /*---the user want to make the partition at the beginning or at the end of free space?---*/
        if (dlgcreate->radioBegin->isChecked()) {
            start = diskview->selPartInfo()->start,
            end = start + dlgcreate->Size();
        } else {
            end = diskview->selPartInfo()->end;
            start = end - dlgcreate->Size();
        }
        
        /*---show a progress dialog for long operation---*/
        InitProgressDialog();

        QP_FileSystemSpec *fsspec = diskview->filesystem->nameToFSSpec(dlgcreate->fileSystemName());
        diskview->libparted->mkpartfs(dlgcreate->type(),
                                      fsspec,
                                      start,
                                      end,
                                      dlgcreate->Label());

        /*---destroy the progress dialog---*/
        DoneProgressDialog();

        /*---refresh diskview widget!---*/
        refreshDiskView();
    }
}

void QP_MainWindow::slotResize() {
    ShowMoveResizeDialog(QTParted::resize);
}

void QP_MainWindow::slotMove() {
    ShowMoveResizeDialog(QTParted::move);
}

void QP_MainWindow::ShowMoveResizeDialog(QTParted::MoveResize moveresize) {
    //FIXME: extended partition can be resized also on the "left"

    QP_PartInfo *partinfo = diskview->selPartInfo();

    if ((partinfo->t_start == -1)
     || (partinfo->t_end   == -1)) {
        QString label = QString(tr("An internal error happened during partition scan.\n"
                                   "This means that you cannot resize or move this "
                                   "partition."));
        QMessageBox::information(this, PROG_NAME, label);

        return ;
    }

    /*---get info about the selected partition---*/
    PedSector StartPart = partinfo->start;
    PedSector EndPart = partinfo->end;
    PedSector GrowStartPart = partinfo->t_start;
    PedSector GrowEndPart = partinfo->t_end;
    PedSector MinPart = partinfo->min_size;

    /*---set the info that were just get---*/
    dlgresize->setGrowStartPartSector(GrowStartPart);
    dlgresize->setGrowEndPartSector(GrowEndPart);
    dlgresize->setEndPartSector(EndPart);
    dlgresize->setStartPartSector(StartPart);
    dlgresize->setMinPartSector(MinPart);

    /*---init of the dialog box used for resize/move a partition---*/
    dlgresize->init_dialog(moveresize);

    /*---show dialog, and get return code (useful for getting "accepted"/"cancel" of dialog---*/
    int code = dlgresize->show_dialog();

    /*---the user clicked ok?---*/
    if (code == QDialog::Accepted) {
        QString label;

        label = QString("<qt><center><b>" + tr("WARNING") + "</b></center>" + "<br>"
                        + tr("Are you sure to process?") + "<br><br>"
                        + "<center>"
                        + tr("LAST CHANCE TO QUIT")
                        + "</center>"
                        + "</qt>");

        /*---are you really sure? :-)---*/
        QMessageBox mb(PROG_NAME,
            label,
            QMessageBox::Information,
            QMessageBox::Yes,
            QMessageBox::No | QMessageBox::Default | QMessageBox::Escape,
            QMessageBox::NoButton);

        /*---no, the user is not sure :( ---*/
        if ( mb.exec() == QMessageBox::No)
            return ;

        /*---show a progress dialog for long operation---*/
        InitProgressDialog();

        /*---call libparted resize/move function!---*/
        bool rc;
        if (moveresize == QTParted::resize)
            rc = diskview->selPartInfo()->resize(dlgresize->_StartPartSector,
                                                 dlgresize->_EndPartSector);
        else
            rc = diskview->selPartInfo()->move(dlgresize->_StartPartSector,
                                               dlgresize->_EndPartSector);

        /*---destroy the progress dialog---*/
        DoneProgressDialog();

        /*---refresh diskview widget!---*/
        refreshDiskView();
    }
}

void QP_MainWindow::InitMenu() {
    actProperty->setEnabled(false);
    actCreate->setEnabled(false);
    actResize->setEnabled(false);
    actMove->setEnabled(false);
    actDelete->setEnabled(false);
    mnuSetSystem->setEnabled(false);
    mnuOperations->setItemEnabled(mnuSetActiveID, false);
}

void QP_MainWindow::InitProgressDialog() {
    /*---initialize the progress dialog and show it---*/
    dlgprogress->init_dialog();
    dlgprogress->show();

    /*---just update GUI---*/
    qApp->processEvents();
}

void QP_MainWindow::DoneProgressDialog() {
    /*---update GUI and disconnect sigTimer---*/
    qApp->processEvents();

    dlgprogress->exec();
}

void QP_MainWindow::slotDelete() {
    QString label = QString("<qt><center><b>" + tr("WARNING") + "</b></center>" + "<br>"
                        + tr("Are you really sure to remove the selected partition?") + "<br><br>"
                        + "<center>"
                        + tr("LAST CHANCE TO QUIT")
                        + "</center>"
                        + "</qt>");

    /*---show a message box that ask confirmation---*/
    QMessageBox mb(PROG_NAME,
        label,
        QMessageBox::Information,
        QMessageBox::Yes,
        QMessageBox::No | QMessageBox::Default | QMessageBox::Escape,
        QMessageBox::NoButton);

    if ( mb.exec() == QMessageBox::No) {
        return ;
    } else {
        /*---call the libparted wrapper to remove the selected partition---*/
        diskview->libparted->rm(diskview->selPartInfo()->num);

        /*---refresh diskview widget!---*/
        refreshDiskView();
    }
}

void QP_MainWindow::slotConfig() {
    /*---init of the configuration dialog---*/
    dlgconfig->init_dialog();

    /*---get the layout---*/
    int layout = settings->layout();

    /*---set the current layout---*/
    dlgconfig->setLayout(layout);

    int code = dlgconfig->show_dialog();
    if (code == QDialog::Accepted) {
        int layout = dlgconfig->layout();
        settings->setLayout(layout);
        diskview->setLayout(layout);

        lstExternalTools->apply();
    } else {
        lstExternalTools->cancel();
    }
}

void QP_MainWindow::slotProperty() {
    /*---make a label with information about the partition selected---*/
    QString label;
    
    QString type;
         if (diskview->selPartInfo()->type == QTParted::primary)
         type.sprintf(tr("primary"));
    else if (diskview->selPartInfo()->type == QTParted::logical)
         type.sprintf(tr("logical"));
    else type.sprintf(tr("extended"));

    label = QString(tr("this is a %1 partition\n"
                    "it start at: %2 and "
                    "end at: %3\n"
                    "it can grow left: %4 and "
                    "right: %5\n"
                    "the filesystem is: %6"))
                            .arg(type)
                            .arg(diskview->selPartInfo()->mb_start())
                            .arg(diskview->selPartInfo()->mb_end())
                            .arg(diskview->selPartInfo()->mb_t_start())
                            .arg(diskview->selPartInfo()->mb_t_end())
                            .arg(diskview->selPartInfo()->fsspec->name());

    QMessageBox::information(this, PROG_NAME, label);
}

void QP_MainWindow::slotWhatsThis() {
    whatsThis();
}

void QP_MainWindow::slotAbout() {
    QString content = QString("<h3>%1 v%2</h3>\n").arg(PROG_NAME).arg(VERSION);
    content += "<br>\n";
    content += "Copyright (C) 2003<br>\n";
    content += "By Vanni Brutto<br>\n";
    content += "zanac@libero.it<br>\n";
    content += "http://qtparted.sourceforge.net/<br>\n";
    content += "<br>\n";
    content += "QTParted is free software, covered by the GNU General Public License, and you are welcome to change it and/or distribute copies of it under certain conditions.<br>\n";
    content += "<br>\n";
    content += "A Partition Magic Clone written in C++ using the Qt GUI Toolkit.<br>\n";
    content += "<br>\n";
    content += "See the AUTHORS file for more info about contributors.<br>\n";

    QP_dlgAbout *w = new QP_dlgAbout(qtparted_xpm, content, this);
    w->setCaption(tr("About"));
    w->exec();
    delete w;
}

void QP_MainWindow::slotAboutQT() {
    QMessageBox::aboutQt(this);
}

void QP_MainWindow::slotSetSystem(QAction *action) {
    QString name = action->text();
    QP_FileSystemSpec *fsspec = diskview->filesystem->nameToFSSpec(name);
    diskview->selPartInfo()->set_system(fsspec);
}

void QP_MainWindow::slotSelectPart(QP_PartInfo *partinfo) {
    /*---check/uncheck the active flag on the menu popup---*/
    mnuOperations->setItemEnabled(mnuSetActiveID, partinfo->canBeActive());
    mnuOperations->setItemChecked(mnuSetActiveID, partinfo->isActive());

    if (diskview->selPartInfo()->fsspec == diskview->filesystem->free()) {
        actCreate->setEnabled(true);
        actResize->setEnabled(false);
        actMove->setEnabled(false);
        actDelete->setEnabled(false);
        mnuSetSystem->setEnabled(false);
    } else {
        bool resize = partinfo->fsspec->resize();
        bool move = partinfo->fsspec->move();
        actCreate->setEnabled(false);
        actDelete->setEnabled(true);
        actMove->setEnabled(move);

        if (partinfo->type == QTParted::extended) {
            actResize->setEnabled(true);
            mnuSetSystem->setEnabled(false);
        }
        else {
            actResize->setEnabled(resize);
            mnuSetSystem->setEnabled(true);
        }

        /*---scan for every menu in the group setSystem---*/
        QAction *p;
        for (p = (QAction *)lstSetSystem.first(); p; p = (QAction *)lstSetSystem.next()) {
            /*---the name match, so select it!---*/
            if (p->text().compare(diskview->selPartInfo()->fsspec->name()) == 0) {
                agSetSystem->blockSignals(true);
                p->setOn(true);
                agSetSystem->blockSignals(false);
            }
        }
    }
    actProperty->setEnabled(true);
}

void QP_MainWindow::slotPopup(QPoint pos) {
    popupmenu()->popup(pos);
}

void QP_MainWindow::slotSelected(QString dev) {
    /*---initialization of menuitem---*/
    InitMenu();
    
    /*---show a progress dialog for long operation---*/
    InitProgressDialog();

    /*---remove all menuitems---*/
    mnuSetSystem->clear();

    /*---clear old lstSetSystem list---*/
    lstSetSystem.clear();

    /*---create the action group---*/
    agSetSystem = new QActionGroup(this, 0);
    agSetSystem->setExclusive(true);
    connect(agSetSystem, SIGNAL(selected(QAction *)),
            this, SLOT(slotSetSystem(QAction *)));

    diskview->setDevice(dev);

    /*---destroy the progress dialog---*/
//    DoneProgressDialog();

    /*---loop into filesystemlist, and add it to combobox if you it has create flag---*/
    QP_FileSystemSpec *p;
    for (p = (QP_FileSystemSpec*)diskview->filesystem->filesystemlist.first(); 
            p; p = (QP_FileSystemSpec*)diskview->filesystem->filesystemlist.next()) {
        if ((p->name().compare("free") != 0)
        &&  (p->name().compare("unknow") != 0)) {
        	QAction *actSetSystem = new QAction(p->name(), QIconSet(p->pixmap()), QString::null/*accell*/,
                                            0, agSetSystem, 0, agSetSystem->isExclusive());
            lstSetSystem.append(actSetSystem);
        }
    }
    agSetSystem->addTo(mnuSetSystem);
    
    /*---closethe progress dialog---*/
    dlgprogress->hide();
}

void QP_MainWindow::slotSetActive() {
    QString oldActive;
    QString newActive;
    QString noactive;
    QString label;
    
    noactive = QString(tr("(No Active on Drive)"));

    if (!diskview->libparted->partActive())
        oldActive = noactive;
    else
        oldActive = diskview->libparted->partActive()->partname();

    bool active = mnuOperations->isItemChecked(mnuSetActiveID);
    if (active)
        newActive = noactive;
    else
        newActive = diskview->selPartInfo()->partname();
    
    label = QString(tr("Currently active partition: %1\n"
                       "Change active partition to: %2\n\n"
                       "Are you sure you want to change the active partition?"))
                                   .arg(oldActive)
                                   .arg(newActive);

    QMessageBox mb(PROG_NAME,
                  label,
                  QMessageBox::Information,
                  QMessageBox::Yes,
                  QMessageBox::No | QMessageBox::Default | QMessageBox::Escape,
                  QMessageBox::NoButton);

    /*---yes, the user is sure :) ---*/
    if (mb.exec() == QMessageBox::Yes) {
        diskview->selPartInfo()->setActive(!active);

        /*---refresh diskview widget!---*/
        refreshDiskView();
    }
}
